/** Teardrop - myFrame.cpp

# Copyright (C) 2006 Olivier COUPELON

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "MyFrame.h"

// Event table for myFrame
BEGIN_EVENT_TABLE(MyFrame, wxFrame)
EVT_BUTTON(wxID_FIND, MyFrame::OnSearchNow)
EVT_TEXT_ENTER(wxID_OK, MyFrame::OnSearchNow)
EVT_MENU(wxID_ABOUT, MyFrame::OnAbout)
EVT_MENU(wxID_EXIT, MyFrame::OnQuit)
EVT_MENU(wxID_OPEN, MyFrame::OnOpen)
EVT_MENU(wxID_SAVEAS, MyFrame::OnSaveAs)
EVT_MENU(wxID_SETUP, MyFrame::OnPref)
EVT_MENU(wxID_DUPLICATE, MyFrame::OnExport)
EVT_COMBOBOX(wxID_VIEW_SORTTYPE, MyFrame::OnOrderBy)
EVT_TEXT_ENTER(wxID_VIEW_SORTTYPE, MyFrame::OnOrderBy)
EVT_TEXT_ENTER(wxID_UP, MyFrame::OnSearchNow)
EVT_BUTTON(wxID_UP, MyFrame::OnMoveUp)
EVT_BUTTON(wxID_DOWN, MyFrame::OnMoveDown)
EVT_BUTTON(wxID_HOME, MyFrame::OnNewCategory) //Not well choosen id, but...
EVT_MENU(wxID_INDEX, MyFrame::OnValidateLinks)
EVT_MENU(wxID_REFRESH, MyFrame::OnUpdateEngines)
END_EVENT_TABLE()


MyFrame::MyFrame(wxWindow* parent, int id, const wxString& title, const wxPoint& pos, const wxSize& size, long style):
    wxFrame(parent, id, title, pos, size, wxDEFAULT_FRAME_STYLE) {
    m_search = "";
    wxString orderChoice[] = 
    {
        _("Relevance"), 
        _("Title"), 
        _("Abstract"), 
        _("Url")
    };
    panelB1             = new wxPanel(this, -1);
    panelB2             = new wxSplitterWindow(this, -1, wxDefaultPosition, wxSize(-1,200));
    panelC3             = new wxPanel(panelB2, -1, wxDefaultPosition,wxDefaultSize);
    //The following two MUST be created before ANY component that goes on top of it. (See wxStaticBox doc).
    sizerB1          	= new wxStaticBoxSizer(wxVERTICAL,panelB1,_("Query"));
    sizerC3	            = new wxStaticBoxSizer(wxVERTICAL,panelC3,_("Engines"));
    panelC1             = new wxPanel(panelB1, -1, wxDefaultPosition,wxDefaultSize);
    panelC2             = new wxPanel(panelB1, -1, wxDefaultPosition,wxDefaultSize);
    panelC4             = new wxPanel(panelB2, -1, wxDefaultPosition,wxDefaultSize);
    panelD1             = new wxPanel(panelC3, -1, wxDefaultPosition,wxDefaultSize);
    panelD2             = new wxPanel(panelC4, -1, wxDefaultPosition,wxDefaultSize,wxRAISED_BORDER);
    
    wxTextCtrlSearch    = new wxTextCtrl(panelC1, wxID_OK, _T(""),wxDefaultPosition,wxSize(400,-1),wxTE_PROCESS_ENTER);
    wxButtonSearch      = new wxBitmapButton(panelC1, wxID_FIND,wxBitmap(search_xpm));
    wxSpinCtrl_1        = new wxSpinCtrl(panelC2, wxID_UP, _T("10"), wxDefaultPosition, wxDefaultSize,wxSP_VERTICAL|wxSP_ARROW_KEYS|wxTE_PROCESS_ENTER,1,1000,10);
    wxSpinText          = new wxStaticText(panelC2,-1,_("Results per engines"),wxDefaultPosition,wxSize(200,-1));
    wxComboText_1       = new wxStaticText(panelD2,-1,_("Order results by : "));
    wxComboBox_1        = new wxComboBox(panelD2, wxID_VIEW_SORTTYPE, _("Relevance"), wxDefaultPosition, wxDefaultSize, 4, orderChoice, wxCB_READONLY|wxCB_SORT|wxNO_BORDER);
    ListResults         = new wxListResults(panelC4, -1);
    wxButtonUp          = new wxBitmapButton(panelD1, wxID_UP,wxBitmap(uparrow_xpm));
    wxButtonDown        = new wxBitmapButton(panelD1, wxID_DOWN,wxBitmap(downarrow_xpm));
    wxButtonCategory    = new wxBitmapButton(panelD1, wxID_HOME,wxBitmap(newcategory_xpm));
    wxEngineTree_1      = new wxEngineTree(panelC3, -1, wxDefaultPosition, wxDefaultSize, wxTR_HIDE_ROOT|wxTR_MULTIPLE|wxTR_HAS_BUTTONS|wxSUNKEN_BORDER|wxTR_HAS_VARIABLE_ROW_HEIGHT|wxTR_DEFAULT_STYLE,this);
    
    SetMenuBar(new MyMenuBar());
    CreateStatusBar();
    SetStatusText(_("Welcome to Teardrop!"));
    
    SetIcon(wxIcon(td_icon_xpm));
    
    CreateToolBar(wxNO_BORDER|wxHORIZONTAL|wxTB_FLAT);
    GetToolBar()->SetToolBitmapSize(wxSize(22,22));
    GetToolBar()->AddTool(wxID_OPEN,_("Open"), wxBitmap(fileopen_xpm),_("Open"));
    GetToolBar()->AddTool(wxID_SAVEAS,_("Save As..."), wxBitmap(filesaveas_xpm),_("Save As..."));
    GetToolBar()->AddSeparator();
    GetToolBar()->AddTool(wxID_DUPLICATE,_("Export..."), wxBitmap(fileexport_xpm),_("Export..."));
    GetToolBar()->Realize();
    
    do_layout();
    
    //Setting proxy settings
    string m_proxyAddress = "";
    string m_proxyPort = "0";
    string m_proxyType = "HTTP";
    xmlFile xf;
    string path;
    if (selectFile::find("config",".xml",path) && xf.openFile(path + "config.xml")) {
        xmlNode *n;
        n = xf.findChildByName(NULL,"proxy-address");
        if (n != NULL) {
            m_proxyAddress = xf.getNodeValue(n);
        }
        n = xf.findChildByName(NULL,"proxy-port");
        if (n != NULL) {
            m_proxyPort = xf.getNodeValue(n);
        }
        n = xf.findChildByName(NULL,"proxy-type");
        if (n != NULL) {
            m_proxyType = xf.getNodeValue(n);
        }
        n = xf.findChildByName(NULL,"maxthread");
        if (n != NULL) {
            tdp.setMaxThread(xf.getNodeValue(n));
        }
        n = xf.findChildByName(NULL,"timeout");
        if (n != NULL) {
            tdp.setTimeout(xf.getNodeValue(n));
        }
    }
    
    tdp.setProxy(m_proxyAddress,m_proxyPort,m_proxyType);
    wxEngineTree_1->SetTdParam(GetTdParam());
    ListResults->SetTdParam(GetTdParam());
}

void MyFrame::do_layout()
{
	wxBoxSizer* sizerA1			= new wxBoxSizer(wxVERTICAL);
	wxBoxSizer* sizerB2			= new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer* sizerC1		    = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer* sizerC2		    = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer* sizerC4			= new wxBoxSizer(wxVERTICAL);
    wxBoxSizer* sizerD1			= new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer* sizerD2			= new wxBoxSizer(wxHORIZONTAL);
	
    sizerD1->Add(wxButtonUp);
    sizerD1->Add(wxButtonDown,0,wxRIGHT,10);
    sizerD1->Add(wxButtonCategory);
    Link(panelD1,sizerD1);
    
    
    sizerD2->Add(wxComboText_1,0,wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxRIGHT,10);
    sizerD2->Add(wxComboBox_1);
    Link(panelD2,sizerD2);

    
    sizerC1->Add(wxTextCtrlSearch, 0, wxALL,5);
    sizerC1->Add(wxButtonSearch, 0, wxALL,5);
    Link(panelC1,sizerC1);
    
    sizerC2->Add(wxSpinCtrl_1, 0, wxALL,5);
    sizerC2->Add(wxSpinText, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);
    Link(panelC2,sizerC2);
    
    sizerB1->Add(panelC1,0,wxEXPAND);
    sizerB1->Add(panelC2,0,wxEXPAND);
    Link(panelB1,sizerB1);
	
    sizerA1->Add(panelB1,0,wxEXPAND);
    
    
    sizerC3->Add(panelD1,0,wxALIGN_RIGHT);
    sizerC3->Add(wxEngineTree_1,1,wxEXPAND);
    Link(panelC3,sizerC3);
    
    
    sizerC4->Add(panelD2,0,wxEXPAND);
    sizerC4->Add(ListResults,1,wxEXPAND);
    Link(panelC4,sizerC4);
	

    panelB2->SplitVertically(panelC3, panelC4,200);
    panelB2->SetMinimumPaneSize(200);
    
    sizerA1->Add(panelB2,1,wxEXPAND);
    
    SetSizerAndFit(sizerA1); //This sets the sizer as the main sizer
    
    wxButtonUp->SetToolTip(_("Move Up"));
    wxButtonDown->SetToolTip(_("Move Down"));
    wxButtonCategory->SetToolTip(_("New Category"));
    wxButtonSearch->SetToolTip(_("Find"));
    
    Layout();
}

void MyFrame::Link(wxPanel *p, wxSizer *s) {
    p->SetAutoLayout(true);
    p->SetSizer(s);
    s->Fit(p);
    s->SetSizeHints(p);
}

void MyFrame::OnSearchNow(wxCommandEvent& event) {
    
    string search = toSTLString(wxTextCtrlSearch->GetValue());
    if (search == "") {
        wxMessageBox(_("Please specify a query !"),_("Error !"),wxOK | wxICON_ERROR, this);
        return;
    }
    
    m_listEngine = wxEngineTree_1->getSelectedName();

    if (!m_listEngine.size()) {
        wxMessageBox(_("Please select at least one engine !"),_("Error !"),wxOK | wxICON_ERROR, this);
        return;
    }    
    
    string lm = toSTLString(wxString::Format(_T("%d"), wxSpinCtrl_1->GetValue()));
    
    tdParam tdp = GetTdParam();
    metaRank *mr = new metaRank(search,m_listEngine, lm, tdp);
    
    SetStatusText(_("Query started..."));
    
    /* Progress bar zone !
    */
    mr->startParsing();
    int max = m_listEngine.size()+1;
    wxProgressDialog dialog(_("Processing your query..."), _("Please wait while the query is being processed..."), max,this, wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_CAN_SKIP | wxPD_CAN_ABORT);
    dialog.Update(0);
    bool skip = false;
    bool cancel = false;
    while (mr->isAlive()) {
        wxMilliSleep(500);
        if (!dialog.Update(mr->howManyFinished(),_T(""),&skip)) {
            cancel = true;
            mr->stop();
        }
        if (skip) mr->stop();
    }
    mr->joinAll();
    if (!cancel) {
        mr->startRanking();
        m_limit = lm;
        m_search = search;
        m_listRow = mr->getResults();
        mr->sort(m_listRow,GetValueComboBox());
        updateList(m_listRow);
        
        wxString msg;
        msg.Printf(_("Query processed, %i results found !"), m_listRow.size());
        SetStatusText(msg);
    } else SetStatusText(_("Query aborted !"));
    //This closes the dialog
    dialog.Update(max);
    delete mr;
    
}

void MyFrame::updateList(list<row> & r) {
    ListResults->Freeze();
    ListResults->DeleteAllItems();
    ListResults->Add(r);
    Refresh();
    ListResults->Thaw();
}

void MyFrame::OnAbout(wxCommandEvent& event) {
    wxString msg;
    msg.Printf(_("Thank you for using Teardrop %s\n\nDistributed by Olivier COUPELON (olivier.coupelon@wanadoo.fr) under GPL\nPowered by %s"),wxT(VERSION), wxVERSION_STRING);
    wxMessageBox(msg, _("About Teardrop"), wxOK | wxICON_INFORMATION, this);
}

void MyFrame::OnQuit(wxCommandEvent& event) {
    // Destroy the frame
    Close();
}

void MyFrame::OnSaveAs(wxCommandEvent& event) {
    if (!m_search.empty()) {
        wxString caption = _("Choose a file");
        wxString wildcard = _("Teardrop files (*.xml)|*.xml");
        wxString defaultDir = wxEmptyString;
        wxString defaultFilename = _("newfile.xml");
        wxFileDialog dialog(this, caption, defaultDir, defaultFilename,wildcard, wxSAVE|wxOVERWRITE_PROMPT);
        if (dialog.ShowModal() == wxID_OK)
        {
            if (!openSave::xmlSave(toSTLString(dialog.GetPath()),m_search,m_listEngine,m_limit,m_listRow)) {
                wxMessageBox(_("Save failed !"),_("Error !"),wxOK | wxICON_ERROR, this);
            }
        }
    }
}

void MyFrame::OnOpen(wxCommandEvent &event) {
    wxString caption = _("Choose a file");
    wxString wildcard = _("Teardrop files (*.xml)|*.xml");
    wxString defaultDir = wxEmptyString;
    wxString defaultFilename = wxEmptyString;
    wxFileDialog dialog(this, caption, defaultDir, defaultFilename,wildcard, wxOPEN|wxFILE_MUST_EXIST);
    if (dialog.ShowModal() == wxID_OK)
    {
        m_listEngine.clear();
        m_listRow.clear();
        if (!openSave::xmlOpen(toSTLString(dialog.GetPath()),m_search,m_listEngine,m_limit,m_listRow)) {
            wxMessageBox(_("Open failed !"),_("Error !"),wxOK | wxICON_ERROR, this);
            return;
        }
        updateList(m_listRow);
    }
}

void MyFrame::OnOrderBy(wxCommandEvent &event) {
    metaRank::sort(m_listRow,GetValueComboBox());
    updateList(m_listRow);
}

string MyFrame::GetValueComboBox() {
    if(wxComboBox_1->GetValue().Cmp(_("Relevance")) == 0) return "number";
    else if(wxComboBox_1->GetValue().Cmp(_("Title")) == 0) return "title";
    else if(wxComboBox_1->GetValue().Cmp(_("Abstract")) == 0) return "abstract";
    else if(wxComboBox_1->GetValue().Cmp(_("Url")) == 0) return "url";
    return "";
}

void MyFrame::OnExport(wxCommandEvent &event) {
    if (!m_search.empty()) {
        wxString caption = _("Choose a file");
        wxString wildcard = _("Web page (*.html)|*.html|CSV files (*.csv)|*.csv");
        wxString defaultDir = wxEmptyString;
        wxString defaultFilename = wxEmptyString;
        wxFileDialog dialog(this, caption, defaultDir, defaultFilename,wildcard, wxSAVE|wxOVERWRITE_PROMPT);
        if (dialog.ShowModal() == wxID_OK)
        {
            switch(dialog.GetFilterIndex()) {
                case 0 :
                    if (!openSave::htmlExport(toSTLString(dialog.GetPath()),m_search,m_listEngine,m_limit,m_listRow)) {
                        wxMessageBox(_("Export failed !"),_("Error !"),wxOK | wxICON_ERROR, this);
                    }
                    break;
                case 1 :
                    if (!openSave::csvExport(toSTLString(dialog.GetPath()),m_listRow)) {
                        wxMessageBox(_("Export failed !"),_("Error !"),wxOK | wxICON_ERROR, this);
                    }
                    break;
                default:
                    break;
            }
            
        }
    }
}

tdParam & MyFrame::GetTdParam() {
    return tdp;
}

void MyFrame::OnMoveDown(wxCommandEvent& event) {
    wxEngineTree_1->OnMoveDown(event);
}

void MyFrame::OnMoveUp(wxCommandEvent& event) {
    wxEngineTree_1->OnMoveUp(event);
}

void MyFrame::OnNewCategory(wxCommandEvent& event) {
    wxEngineTree_1->OnNewCategory(event);
}

void MyFrame::OnPref(wxCommandEvent &event) {
    xmlFile xf;
    string path;
    wxString m_browser;
    if (selectFile::find("config",".xml",path) && xf.openFile(path + "config.xml")) {
        xmlNode *n;
        n = xf.findChildByName(NULL,"browser");
        if (n != NULL) {
            m_browser = toWXString(xf.getNodeValue(n));
        }
    
        wxPrefDialog dialog(NULL, -1, _("Preferences"));
        dialog.setBrowser(m_browser);
        dialog.setProxyAddress(toWXString(tdp.getProxyAddress()));
        dialog.setProxyPort(wxString::Format(_T("%d"),tdp.getProxyPort()));
        dialog.setProxyHttp(tdp.getProxyType() == CURLPROXY_HTTP);
        dialog.setMaxThread(wxString::Format(_T("%d"),tdp.getMaxThread()));
        dialog.setTimeout(wxString::Format(_T("%d"),tdp.getTimeout()));
        if (dialog.ShowModal() == wxID_OK) {
            m_browser = dialog.getBrowser();
            string proxyAddress = toSTLString(dialog.getProxyAddress());
            string proxyPort = toSTLString(dialog.getProxyPort());
            string proxyType = (dialog.getProxyHttp()?"HTTP":"SOCKS5");
            
            xf.setNodeContent(NULL,"browser",toSTLString(m_browser));
            xf.setNodeContent(NULL,"proxy-address",proxyAddress);
            xf.setNodeContent(NULL,"proxy-port",proxyPort);
            xf.setNodeContent(NULL,"proxy-type",proxyType);
            xf.setNodeContent(NULL,"timeout",toSTLString(dialog.getTimeout()));
            xf.setNodeContent(NULL,"maxthread",toSTLString(dialog.getMaxThread()));
            
            tdp.setProxy(proxyAddress,proxyPort,proxyType);
            wxEngineTree_1->SetTdParam(tdp);
            
            string out = selectFile::getHomeDirectory() + "config.xml";
            xf.saveDocTo(out);
        }
    }
}

void MyFrame::OnValidateLinks(wxCommandEvent& event) {
    threadPool<validPages> tp(false,tdp.getMaxThread());
    list<bool> pages;
    for (list<row>::iterator lrit = m_listRow.begin(); m_listRow.end() != lrit; ++lrit) {
        string url = lrit->getField("url");
        if (url != "") {
            pages.push_back(true);
            validPages *vp = new validPages(url,&(*pages.rbegin()),tdp);
            tp.createThread(vp);
        }
    }
    int max = pages.size()+1;
    wxProgressDialog dialog(_("Verifying links..."), _("Please wait while links are being verified..."), max,this, wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_CAN_ABORT);
    dialog.Update(0);
    while (tp.isAlive()) {
        wxMilliSleep(500);
        if (!dialog.Update(tp.howManyFinished(),_T(""))) {
            tp.cancelAll();
            return;
        }
    }
    tp.joinAll();
    list<row> validated;
    for (list<row>::iterator lrit = m_listRow.begin(); m_listRow.end() != lrit; ++lrit) {
        string url = lrit->getField("url");
        if (url != "") {
            if (pages.front()) validated.push_back(*lrit);
            pages.pop_front();
        }
    }
    
    m_listRow = validated;
    wxString msg;
    msg.Printf(_("Links have been verified, %i results remaining !"), m_listRow.size());
    SetStatusText(msg);
    updateList(m_listRow);
    
    dialog.Update(max);
    
}

void MyFrame::OnUpdateEngines(wxCommandEvent& event) {
    wxEngineTree_1->UpdateEngines();
}
