/** Teardrop - MyFrame.h

@brief Deals with the GUI

@author Olivier COUPELON (olivier.coupelon@wanadoo.fr)
@date 19/08/2006

# Copyright (C) 2006 Olivier COUPELON

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef MYFRAME_H
#define MYFRAME_H

#include <wx/wx.h>
#include <wx/progdlg.h>
#include <wx/splitter.h>
#include <wx/spinctrl.h>

#include "metaRank.h"
#include "MyMenuBar.h"
#include "row.h"
#include "wxEngineTree.h"
#include "stringsConversions.h"
#include "wxListResults.h"
#include "getHttp.h"
#include "openSave.h"
#include "tdParam.h"
#include "wxPrefDialog.h"
#include "validPages.h"

#include "fileopen.xpm"
#include "filesaveas.xpm"
#include "fileexport.xpm"
#include "downarrow.xpm"
#include "uparrow.xpm"
#include "newcategory.xpm"
#include "search.xpm"

#ifndef __WXMSW__
    #include "td_icon.xpm"
#else
    #include "td_icon_msw.xpm"
#endif


class MyFrame: public wxFrame {
public:

    MyFrame(wxWindow* parent, int id, const wxString& title, const wxPoint& pos=wxDefaultPosition, const wxSize& size=wxDefaultSize, long style=wxDEFAULT_FRAME_STYLE);
    ~MyFrame() {};
    void Link(wxPanel *p, wxSizer *s);

protected:
    //wxButton *wxButtonSearch;
    wxBitmapButton *wxButtonUp, *wxButtonDown, *wxButtonCategory,*wxButtonSearch;
    wxTextCtrl *wxTextCtrlSearch;
    //wxPanel *wxPanel_1,*wxPanel_2,*wxPanel_3,*wxPanel_4,*wxPanel_5,*wxPanel_6,*wxPanel_7,*wxPanel_8;
    
    wxEngineTree *wxEngineTree_1;
    wxListResults *ListResults;
    wxSpinCtrl *wxSpinCtrl_1;
    wxMenuBar *wxMenuBar_1;
    wxComboBox *wxComboBox_1;
    wxStaticText *wxSpinText,*wxComboText_1;
    
    /* 'A' for back level, growing to top level (B,C,D...)
       The number counts the panels on the same level */
    wxPanel *panelB1;
    wxSplitterWindow *panelB2;
    wxPanel *panelC1,*panelC2,*panelC3,*panelC4;
    wxPanel *panelD1,*panelD2,*panelD3;
    wxStaticBoxSizer *sizerB1,*sizerC3;

    void updateList(list<row> &);
    string GetValueComboBox();

    tdParam & GetTdParam();

    //Events   
    void OnSearchNow(wxCommandEvent &event);   
    void OnAbout(wxCommandEvent& event);
    void OnQuit(wxCommandEvent& event);
    void OnOpen(wxCommandEvent &event);
    void OnSaveAs(wxCommandEvent &event);
    void OnExport(wxCommandEvent &event);
    void OnOrderBy(wxCommandEvent &event);
    void OnMoveDown(wxCommandEvent& event);
    void OnMoveUp(wxCommandEvent& event);
    void OnNewCategory(wxCommandEvent& event);
    void OnPref(wxCommandEvent &event);
    void OnValidateLinks(wxCommandEvent& event);
    void OnUpdateEngines(wxCommandEvent& event);

private:
    void do_layout();
    list<row> m_listRow;
    list<string> m_listEngine;
    string m_search;
    string m_limit;

    tdParam tdp;

    DECLARE_EVENT_TABLE()
};


#endif
