/** Teardrop - wxListResults.cpp

# Copyright (C) 2006 Olivier COUPELON

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "wxListResults.h"
#include "page.xpm"

BEGIN_EVENT_TABLE(wxListResults, wxVListBox)
    EVT_LEFT_DCLICK(wxListResults::OnBrowse)
    EVT_RIGHT_DOWN(wxListResults::OnPopupMenu)
    EVT_MENU(wxID_OPEN, wxListResults::OnRightBrowse)
    EVT_MENU(wxID_COPY, wxListResults::OnCopy)
END_EVENT_TABLE()

wxListResults::wxListResults(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name):
wxVListBox(parent,id,pos,size,style,name) {
    SetItemCount(0);
    SetSelectionBackground(MYRESULTSCOLOR);
    //Detect the default browser
    m_browser = _T("");
    xmlFile xf;
    string path;
    if (selectFile::find("config",".xml",path) && xf.openFile(path + "config.xml")) {
        xmlNode *n;
        n = xf.findChildByName(NULL,"browser");
        if (n != NULL) {
            m_browser = toWXString(xf.getNodeValue(n));
        }
    }
    
    m_ddMenu = new wxMenu();
    m_ddMenu->Append(wxID_OPEN,_("Open in Browser"));
    m_ddMenu->Append(wxID_COPY,_("Copy Url"));
}

wxListResults::~wxListResults() {
}

void wxListResults::Add(list<row> & lr) {
    m_vRow.insert(m_vRow.end(),lr.begin(),lr.end());
    SetItemCount(m_vRow.size());
}

void wxListResults::DeleteAllItems() {
    DeselectAll();
    SetItemCount(0);
    Clear();
    m_vRow.clear();
}

void wxListResults::OnDrawItem(wxDC& dc, const wxRect& rect, size_t n) const {
    static map<string,rawContainer *> imgLoaded;
    static set<string> imgQueued;
    static threadPool<rawUrl> thread(true,tdp.getMaxThread());
    
    if (n >= m_vRow.size()) return;
    //Setting this font is required for me under wxGTK2
    wxFont f = GetFont();
    f.SetStyle(wxFONTSTYLE_NORMAL);
    dc.SetFont(f);
    
    wxColour defaultColor = dc.GetTextForeground();
    
    dc.DrawBitmap(wxBitmap(page_xpm),rect.x+1,rect.y+1,true);
    wxRect ttl(rect.x + 25,rect.y,rect.width-125,dc.GetCharHeight());
    
        
    /******************** Engines
    */
    list<string> leng = m_vRow[n].getEngine();
    string engines = "";
    //We use count here to prevent displaying more than 3 engines.
    for (list<string>::iterator lsit = leng.begin(); lsit != leng.end(); ++lsit) {
        if (lsit != leng.begin()) engines += "/";
        engines += *lsit;
    }
    f.SetStyle(wxFONTSTYLE_ITALIC);
    dc.SetFont(f);
    dc.SetTextForeground(wxColour(_T("GREY")));
    wxString sEngines = TruncToSize(dc,engines,150);
    wxCoord w,h;
    dc.GetTextExtent(sEngines,&w,&h);
    dc.DrawText(sEngines,rect.x + rect.width - w - 1,rect.y);
    f.SetStyle(wxFONTSTYLE_NORMAL);
    dc.SetFont(f);
    dc.SetTextForeground(defaultColor);
    
    /******************** Title
    */
    f.SetWeight(wxFONTWEIGHT_BOLD);
    f.SetUnderlined(true);
    dc.SetFont(f);
    dc.DrawText(TruncToSize(dc,m_vRow[n].getField("title"),rect.width-30-w),rect.x + 25,rect.y);
    f.SetUnderlined(false);
    f.SetWeight(wxFONTWEIGHT_NORMAL);
    dc.SetFont(f);
    
    /******************** Image
    */
    int urlLine = dc.GetCharHeight();
    string image = m_vRow[n].getField("img");
    int imageWidth = 0;
    if (image != "") {
        if (imgLoaded.find(image) != imgLoaded.end()) {
            //Display image
            urlLine = 3*dc.GetCharHeight();
            rawContainer *r = imgLoaded.find(image)->second;
            wxMemoryInputStream wxMis(r->getContent(),r->getLength());
            wxImage wxI(wxMis);
            int imageHeight = rect.height-dc.GetCharHeight()-4;
            imageWidth = (int) (wxI.GetWidth()/(double) wxI.GetHeight()*imageHeight);
            wxI.Rescale(imageWidth,imageHeight);
            dc.DrawRectangle(rect.x + 25, rect.y + dc.GetCharHeight(), imageWidth+2, rect.height-dc.GetCharHeight()-2);
            dc.DrawBitmap(wxBitmap(wxI),rect.x+26,rect.y + dc.GetCharHeight()+1);
            imageWidth += 5;
        } else {
            threadPool<rawUrl>::lock(thread);
            if (imgQueued.find(image) == imgQueued.end()) {
                imgQueued.insert(image);
                threadPool<rawUrl>::unlock(thread);
                thread.createThread(new rawUrl(image,&imgLoaded,tdp,this));
            } else threadPool<rawUrl>::unlock(thread);
        }
    }
    
    /******************** Abstract
    */
    
    if (m_vRow[n].getField("abstract") != "") {
        wxString complete = toWXString(m_vRow[n].getField("abstract"));
        wxString first = TruncToSize(dc,complete,rect.width-imageWidth-25,false);
        wxString second = TruncToSize(dc,complete.Mid(first.Length()),rect.width-imageWidth-25);
        dc.DrawText(first,rect.x + imageWidth + 25,rect.y + dc.GetCharHeight());
        dc.DrawText(second,rect.x + imageWidth + 25,rect.y + 2*dc.GetCharHeight());
        if (imageWidth == 0)
            urlLine*=3;
    }
    
    /******************** Url
    */
    
    dc.SetTextForeground(*wxBLUE);
    dc.DrawText(TruncToSize(dc,m_vRow[n].getField("url"),rect.width-imageWidth-25),rect.x + imageWidth + 25,rect.y + urlLine);
    dc.SetTextForeground(defaultColor);
}

void wxListResults::OnDrawSeparator(wxDC& dc , wxRect& rect , size_t n ) const {
    if (n >= m_vRow.size()) return;
    if (GetSelection() != n) {
        dc.SetPen(*wxGREY_PEN);
        rect.height--;
        dc.DrawLine(rect.x,rect.y+rect.height,rect.x+rect.width,rect.y+rect.height);
    }
   
}

wxCoord wxListResults::OnMeasureItem(size_t n) const {
    if (n >= m_vRow.size()) return 0;
    wxCoord w,h,descent;
    GetTextExtent(_T("A"), &w, &h, &descent);
    if (m_vRow[n].getField("abstract") == "" && m_vRow[n].getField("img") == "") return (h+descent)*2;
    return (h+descent)*4;
}

wxString wxListResults::TruncToSize(wxDC& dc, string s, int size, bool point) const {
    return TruncToSize(dc,toWXString(s),size,point);
}

wxString wxListResults::TruncToSize(wxDC& dc, wxString out, int size, bool point) const {    
    wxArrayInt aSize;
    dc.GetPartialTextExtents(out,aSize);
    
    //Should use dichotomy !
    for(int i = aSize.GetCount(); i > 0 ; --i) {
        if (aSize[i-1] < size) {
            if (i > 3 && i < aSize.GetCount()-1 && point) {
                return out.Left(i-3) + _T("...");
            }
            return out.Left(i);
        }
    }
    return _T("");
}

void wxListResults::OnPopupMenu(wxMouseEvent& event) {
    if (GetSelection() != -1)
        PopupMenu(m_ddMenu);
}

void wxListResults::OnBrowse(wxMouseEvent& event) {
    Browse();
}

void wxListResults::OnRightBrowse(wxCommandEvent& event) {
    Browse();
}

void wxListResults::Browse() {
    if (!m_browser.IsEmpty() && GetSelection() != -1) {
       wxExecute(toWXString(regExp::replaceAll(toSTLString(m_browser),"%b",m_vRow[GetSelection()].getField("url"))), FALSE);
    }
}

void wxListResults::OnCopy(wxCommandEvent& event) {
    if (GetSelection() != -1 && wxTheClipboard->Open()) {
        wxTheClipboard->SetData( new wxTextDataObject(toWXString(m_vRow[GetSelection()].getField("url"))) );
        wxTheClipboard->Close();
    }
}

void wxListResults::SetTdParam(tdParam & t) {
    tdp = t;
}
