/** Teardrop - wxListResults.h

@brief This class handles a list of answers

@author Olivier COUPELON (olivier.coupelon@wanadoo.fr)
@date 19/08/2006

# Copyright (C) 2006 Olivier COUPELON

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef WXLISTRESULTS_H
#define WXLISTRESULTS_H

#include <wx/wx.h>
#include <wx/vlbox.h>
#include <wx/clipbrd.h>
#include <wx/mstream.h>
#include <wx/image.h>

#include <iostream>
#include <vector>
#include <map>
#include <set>

#include "xmlFile.h"
#include "selectFile.h"
#include "row.h"
#include "stringsConversions.h"
#include "regExp.h"
#include "threadPool.h"
#include "rawContainer.h"
#include "rawUrl.h"
#include "tdParam.h"

#define MYRESULTSCOLOR wxColour(210,210,225)

using namespace std;

class wxListResults : public wxVListBox {
public:
    wxListResults(wxWindow* parent, wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = 0, const wxString& name = wxVListBoxNameStr);
    ~wxListResults();

    void Add(list<row> &);
    void DeleteAllItems();
    void OnDrawItem(wxDC& dc, const wxRect& rect, size_t n) const;
    void OnDrawSeparator(wxDC& dc , wxRect& rect , size_t n ) const;
    wxCoord OnMeasureItem(size_t n) const;
    void OnBrowse(wxMouseEvent&);
    void OnPopupMenu(wxMouseEvent&);
    void OnRightBrowse(wxCommandEvent& event);
    void OnCopy(wxCommandEvent&);
    void Browse();
    void SetTdParam(tdParam & t);

private:
    wxString TruncToSize(wxDC&, string, int, bool = true) const;
    wxString TruncToSize(wxDC&, wxString, int, bool = true) const;

    wxString m_browser;
    vector<row> m_vRow;
    wxMenu *m_ddMenu;

    tdParam tdp;

    DECLARE_EVENT_TABLE()
};

#endif
