/** Teardrop - wxPrefDialog.h

@brief This class manages the Preferences.

@author Olivier COUPELON (olivier.coupelon@wanadoo.fr)
@date 09/10/2006

# Copyright (C) 2006 Olivier COUPELON

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef WXPREFDIALOG
#define WXPREFDIALOG

#include <wx/wx.h>
#include <wx/dialog.h>
#include <wx/statline.h>
#include <wx/valgen.h>

/*!
 * Control identifiers
 */
enum {
    ID_BROWSER = 10000,
    ID_PROXY_ADDRESS = 10001,
    ID_PROXY_PORT = 10002,
    ID_PROXY_SOCKS5 = 10003,
    ID_PROXY_HTTP = 10006,
    ID_RESET = 10006,
    ID_THREADNUM = 10007,
    ID_TIMEOUT = 10008
};

class wxPrefDialog: public wxDialog
{
    DECLARE_CLASS(wxPrefDialog)
    DECLARE_EVENT_TABLE()
    
public:
    wxPrefDialog();
    wxPrefDialog( wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& caption = wxT("Preferences"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxCAPTION|wxSTAY_ON_TOP );
    
    bool Create( wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& caption = wxT("Preferences"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxCAPTION|wxSTAY_ON_TOP );

    void CreateControls();

    wxString getBrowser() { return m_browser; }
    void setBrowser(wxString b) {m_browser = b; }
    wxString getProxyAddress() { return m_proxyAddress; }
    void setProxyAddress(wxString a) {m_proxyAddress = a; }
    wxString getProxyPort() { return m_proxyPort; }
    void setProxyPort(wxString p) {m_proxyPort = p; }
    bool getProxyHttp() { return m_proxyHttp; }
    void setProxyHttp(bool h) { m_proxyHttp = h; m_proxySocks5 = !h; }
    wxString getTimeout() { return m_timeout; }
    wxString getMaxThread() { return m_threadMax; }
    void setTimeout(wxString t) { m_timeout = t; }
    void setMaxThread(wxString t) { m_threadMax = t; }
    
    void OnResetClick(wxCommandEvent &event);
    
protected:
    wxString m_browser;
    wxString m_proxyAddress;
    wxString m_proxyPort;
    wxString m_threadMax;
    wxString m_timeout;
    bool m_proxyHttp,m_proxySocks5;
};

#endif
