/** Teardrop - metaRank.cpp

# Copyright (C) 2006 Olivier COUPELON

# This file is part of Teardrop.
#
# Teardrop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Teardrop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Teardrop; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "metaRank.h"

metaRank::metaRank(string q, list<string> & e, string l,tdParam & t, string m) {
    query = q;
    engines = e;
    limit = atol(l.c_str());
    global_results = new list<engineResults>;
    method = m;
    tdp = t;
    threads.setMaxThread(tdp.getMaxThread());
}

metaRank::~metaRank() {
    delete global_results;
}

void metaRank::startParsing() {
    pageParser *p;
    for(list<string>::iterator eng = engines.begin(); engines.end() != eng; ++eng) {
        p = new pageParser(query,*eng,limit,global_results, &threads, tdp);
        pplist.push_back(p);
        threads.createThread(&(*p));
    }
}

bool metaRank::isAlive() {
    return threads.isAlive();
}

void metaRank::joinAll() {
    threads.joinAll();
    /*for(list<pageParser *>::iterator ppit = pplist.begin(); ppit != pplist.end(); ++ppit)
        delete *ppit;*/
}

int metaRank::howManyFinished() {
    return threads.howManyFinished();
}

void metaRank::stop() {
    threads.cancelAll();
}

void metaRank::startRanking() {
    if(method == "simple")
        simpleRank();
    else
        cerr << "Unknown rank method" << endl;
}

void metaRank::toString() {
    for (list<row>::iterator lrit = ranked_results.begin(); ranked_results.end() != lrit; ++lrit) {
        lrit->toString();
    }
}

void metaRank::toString(string page, string d) {
    string p;   
    for (list<row>::iterator lrit = ranked_results.begin(); ranked_results.end() != lrit; ++lrit) {
        p = page;
        list<string> ls = lrit->getEngine();
        string engines = "";
        for(list<string>::iterator lsit = ls.begin(); lsit != ls.end(); ++lsit) {
            if (engines != "") engines += d;
            engines += *lsit;
        }
        //cout << p << endl;      
        p = regExp::replaceAll(p,"{num}",metaRank::intToString(lrit->getNum()));
        p = regExp::replaceAll(p,"{engines}",engines);
        p = regExp::replaceAll(p,"\\\\n","\n");
        p = regExp::replaceAll(p,"\\\\t","\t");

        regExp r("\\{([^\\}]*)\\}");  // Regexp : {[^}]*}
        for(r.newPage(p); !r.endOfMatch(); r.next()) {
            p = regExp::replaceAll(p,"{" + r.getMatch(1) + "}",lrit->getFields()[r.getMatch(1)]);
        }
        cout << p;
    }
}

list<row> & metaRank::getResults() {
    return ranked_results;
}

int metaRank::getResultsNumber(int old) {
    int nw = global_results->size() + ranked_results.size();
    if (nw > old) return nw;
    return old;
}

void metaRank::simpleRank() {
    bool empty = false;
    //int count = 0;
    while (!empty) {
        empty = true;
        for (list<engineResults>::iterator erit = global_results->begin(); global_results->end() != erit; ++erit) {
            if (erit->results.size() > 0) {
                list<row>::iterator lri;
                if ((lri = find(ranked_results.begin(),ranked_results.end(), erit->results.front())) != ranked_results.end()) {
                    lri->addEngine(erit->results.front().getEngine());
                    //count++;               
                } else {
                    row l = erit->results.front();
                    l.setNum(ranked_results.size());
                    ranked_results.push_back(l);
                }
                empty = false;
                erit->results.pop_front();
            }
        }
    }
    //cerr << "Simple rank removed " << count << " results" << endl;   
}

string metaRank::intToString(int x) {
    std::ostringstream o;
    o << x;
    return o.str();
}

void metaRank::sort(list<row> & lr,string field) {
    lr.sort(compareRow(field));
}
